;------------------------------------------------------------------------
;- Written by: Bazz
;-    This code introduces the programmer to the VBLANK interrupt, and
;-    teaches him/her its powers. This should be pretty simple & easy
;-    to go through.
;------------------------------------------------------------------------

;== Include MemoryMap, HeaderInfo, and interrupt Vector table ==
.INCLUDE "header.inc"

;== Include Library Routines ==
.INCLUDE "InitSNES.asm"
.INCLUDE "LoadGraphics.asm"

;== EQUates ==
.ENUM $600
angle		dw
sx			dw
sy			dw
A			dw
B			dw
C			dw
D			dw
.ENDE

;==========================================
; Main Code
;==========================================
.BANK 0 SLOT 0
.ORG 0
.SECTION "MainCode"

Start:
    InitSNES
    rep #$30
    sep #$20
    
    stz angle
    
    ; Load Tile data to VRAM
    stz $2115
    ldx #$0000
    stx $2116
    ldx #$1800
    stx $4300
    ldx #Map
    stx $4302
    lda #:Map
    sta $4304
    ldy #$4000
    sty $4305
    
    lda #$01
    sta $420B
    
    lda #$80
    sta $2115
    ldx #$0000
    stx $2116
    ldx #$1900
    stx $4300
    ldx #Tiles
    stx $4302
    lda #:Tiles
    sta $4304
    ldy #$1980
    sty $4305
    
    lda #$01
    sta $420B

    LoadPalette BG_Palette, 0, 20 

    lda #$07
    sta $2105           ; Set Video mode 7, 8x8 tiles

    lda #$00            ; Set BG1's Tile Map offset to $0000 (Word address)
    sta $2107           ; And the Tile Map size to 32x32

    stz $210B           ; Set BG1's Character VRAM offset to $0000 (word address)
    
    lda #$80			; Color 0 Repeated off screen
    sta $211a
    
    lda #$01
    stz $211b           ; mode 7 matrix parameter A register (low)
	sta $211b           ; Mode 7 matrix parameter A register (high)
	lda #$00
	stz $211c           ; Mode 7 matrix parameter B register (low)
	sta $211c           ; Mode 7 matrix parameter B register (high)
	lda #$00
	stz $211d           ; Mode 7 matrix parameter C register (low)
	sta $211d           ; Mode 7 matrix parameter C register (high)
	lda #$01
	stz $211e           ; Mode 7 matrix parameter D register (low)
	sta $211e           ; Mode 7 matrix parameter D register (high)
    
    lda #$2				; Store $200 (#512, which will center on the whole map which is 1024x1024)
	stz $211f           ; Mode 7 center position X register (low)
	sta $211f           ; Mode 7 center position X register (high)
	lda #$2
	stz $2120           ; Mode 7 center position Y register (low)
	sta $2120           ; Mode 7 center position Y register (high)
	
	rep #$20			; Scroll X and Y to center view on Map
	lda #($200-$80)		; Scroll and center the middle of the map
	sep #$20
	sta $210d
	xba
	sta $210d
	xba
	clc
	adc #$10
	sta $210e
	xba
	sta $210e
	
	ldx #$0a00
	stx sx
	stx sy

    lda #$01            ; Enable BG1
    sta $212C
    
    lda #$80
    sta $4200       	; Enable NMI
    
    lda #$0F
    sta $2100           ; Turn on screen, full Brightness
	
Infinity:
	WAI
	
_done:
    JMP Infinity

;============================================================================
VBlank:
	rep #$10
    sep #$20        ; A/mem=8 bit
    
    lda $4210       ; Clear NMI flag
    
    lda #0			; Set high byte of A to 0
    xba
    
    lda angle
    ina
    sta angle		; Inc angle when A is 8bits so it loops after 255
    rep #$20
	tax				; Pass angle to X for indexing into the sincos table
	
	; Calculate B and C (sin's)
	; B
	lda sx
	sep #$20
	sta $211b
	xba
	sta $211b
	lda sincos.w,X	;sin (x)
	eor #$FF		; (Make negative)
	ina				; (Make negative)
	sta $211c
	ldy $2135
	sty B
	; C
	lda sy			; low byte scale_y
	sta $211b
	lda sy+1		; high byte scale_y
	sta $211b
	lda sincos.w,X	; sin(x)
	sta $211c
	ldy $2135
	sty C
	
	lda #0			; make high byte of A = 0
	xba				;
	txa
	clc
    adc #64			; Make the index point to the cos value
    rep #$20
    tax
	
	; Calculate A and D (cos's)
	; A
	lda sx
	sep #$20
	sta $211b
	xba
	sta $211b
	lda sincos.w,X	; Cos (x)
	sta $211c
	ldy $2135
	sty A
	; D
	lda sy
	sta $211b
	lda sy+1
	sta $211b
	lda sincos.w,X	; Cos (x)
	sta $211c
	ldy $2135
	sty D
	
	; Store results as Matrix Parameters
	lda A
	sta $211b
	lda A+1
	sta $211b
	
	lda B
	sta $211c
	lda B+1
	sta $211c
	
	lda C
	sta $211d
	lda C+1
	sta $211d
	
	lda D
	sta $211e
	lda D+1
	sta $211e

    RTI
;============================================================================
sincos:
	.include "sincos.inc"
.ENDS
;============================================================================

.BANK 1 SLOT 0
.ORG 0
.SECTION "CharacterData"
Map:
	.incbin "gfx/checker.mp7"

Tiles:
    .incbin "gfx/checker.pc7"

BG_Palette:
    .incbin "gfx/checker.clr"

.ENDS
